/*
 * Copyright (C) 2014 desrever <desrever at nubits.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.nubits.nubot.models;

/**
 *
 * @author desrever <desrever at nubits.com>
 */
import java.util.ArrayList;
import java.util.logging.Logger;
import org.json.JSONException;

/**
 *
 * @author advanced
 */
public class CryptoPegOptionsJSON {

    private static final Logger LOG = Logger.getLogger(CryptoPegOptionsJSON.class.getName());
    //Compulsory settings ----------------------------
    private String mainFeed;
    private ArrayList<String> backupFeedNames;
    //Optional settings with a default value  ----------------------------
    private int refreshTime;
    private double wallchangeTreshold, priceOffset, distanceTreshold;
    private int emergencyTimeout;

    /**
     *
     * @param refreshTime
     * @param wallchangeTreshold
     * @param priceOffset
     * @param distanceTreshold
     * @param mainFeed
     * @param backupFeedNames
     * @param emergencyTimeout
     */
    public CryptoPegOptionsJSON(int refreshTime, double wallchangeTreshold, double priceOffset, double distanceTreshold, String mainFeed, ArrayList<String> backupFeedNames, int emergencyTimeout) {
        this.refreshTime = refreshTime;
        this.wallchangeTreshold = wallchangeTreshold;
        this.priceOffset = priceOffset;
        this.distanceTreshold = distanceTreshold;
        this.mainFeed = mainFeed;
        this.backupFeedNames = backupFeedNames;
        this.emergencyTimeout = emergencyTimeout;
    }

    /**
     *
     * @param optionsJSON
     * @return
     */
    public static CryptoPegOptionsJSON create(org.json.JSONObject optionsJSON) {
        OptionsJSON options = null;
        try {
            //First try to parse compulsory parameters

            String mainFeed = (String) optionsJSON.get("main-feed");


            ArrayList<String> backupFeedNames = new ArrayList<>();
            org.json.JSONObject dataJson = (org.json.JSONObject) optionsJSON.get("backup-feeds");

            //Iterate on backupFeeds

            String names[] = org.json.JSONObject.getNames(dataJson);
            if (names.length <= 2) {
                LOG.severe("The bot requires at least two backup data feeds to run");
                System.exit(0);
            }
            for (int i = 0; i < names.length; i++) {
                try {
                    org.json.JSONObject tempJson = dataJson.getJSONObject(names[i]);
                    backupFeedNames.add((String) tempJson.get("name"));
                } catch (JSONException ex) {
                    LOG.severe(ex.getMessage());
                    System.exit(0);
                }
            }

            //Then parse optional settings. If not use the default value declared here

            int refreshTime = 124;
            double wallchangeTreshold = 3;
            double priceOffset = 0;
            double distanceTreshold = 10;
            int emergencyTimeout = 60;



            if (optionsJSON.has("wallchange-treshold")) {
                wallchangeTreshold = new Double((optionsJSON.get("wallchange-treshold")).toString());
            }


            if (optionsJSON.has("price-offset")) {
                priceOffset = new Double((optionsJSON.get("price-offset")).toString());
            }

            if (optionsJSON.has("price-distance-threshold")) {
                distanceTreshold = new Double((optionsJSON.get("price-distance-threshold")).toString());
            }


            if (optionsJSON.has("refresh-time")) {
                refreshTime = new Integer((optionsJSON.get("refresh-time")).toString());
            }


            if (optionsJSON.has("emergency-timeout")) {
                emergencyTimeout = (int) optionsJSON.get("emergency-timeout");
            }


            return new CryptoPegOptionsJSON(refreshTime, wallchangeTreshold, priceOffset, distanceTreshold, mainFeed, backupFeedNames, emergencyTimeout);
        } catch (JSONException ex) {
            LOG.severe(ex.getMessage());
            System.exit(0);
        }
        return null; //never reached
    }

    /**
     *
     * @return
     */
    public int getRefreshTime() {
        return refreshTime;
    }

    /**
     *
     * @param refreshTime
     */
    public void setRefreshTime(int refreshTime) {
        this.refreshTime = refreshTime;
    }

    /**
     *
     * @return
     */
    public double getWallchangeTreshold() {
        return wallchangeTreshold;
    }

    /**
     *
     * @param wallchangeTreshold
     */
    public void setWallchangeTreshold(double wallchangeTreshold) {
        this.wallchangeTreshold = wallchangeTreshold;
    }

    /**
     *
     * @return
     */
    public double getPriceOffset() {
        return priceOffset;
    }

    /**
     *
     * @param priceOffset
     */
    public void setPriceOffset(double priceOffset) {
        this.priceOffset = priceOffset;
    }

    /**
     *
     * @return
     */
    public double getDistanceTreshold() {
        return distanceTreshold;
    }

    /**
     *
     * @param distanceTreshold
     */
    public void setDistanceTreshold(double distanceTreshold) {
        this.distanceTreshold = distanceTreshold;
    }

    /**
     *
     * @return
     */
    public String getMainFeed() {
        return mainFeed;
    }

    /**
     *
     * @param mainFeed
     */
    public void setMainFeed(String mainFeed) {
        this.mainFeed = mainFeed;
    }

    /**
     *
     * @return
     */
    public ArrayList<String> getBackupFeedNames() {
        return backupFeedNames;
    }

    /**
     *
     * @param backupFeedNames
     */
    public void setBackupFeedNames(ArrayList<String> backupFeedNames) {
        this.backupFeedNames = backupFeedNames;
    }

    /**
     *
     * @return
     */
    public int getEmergencyTimeout() {
        return emergencyTimeout;
    }

    /**
     *
     * @param emergencyTimeoutMinutes
     */
    public void setEmergencyTimeoutMinutes(int emergencyTimeoutMinutes) {
        this.emergencyTimeout = emergencyTimeoutMinutes;
    }

    @Override
    public String toString() {
        return "CryptoPegOptionsJSON [" + "backupFeedNames " + backupFeedNames + " " + "distanceTreshold " + distanceTreshold + " " + "emergencyTimeoutMinutes " + emergencyTimeout + " " + "mainFeed " + mainFeed + " " + "priceOffset " + priceOffset + " " + "refreshTime " + refreshTime + " " + "wallchangeTreshold " + wallchangeTreshold + "]";
    }

    String toHtmlString() {
        return "CryptoPegOptionsJSON : <br>" + "backupFeedNames " + backupFeedNames + " <br>" + "distanceTreshold " + distanceTreshold + "<br> " + "emergencyTimeoutMinutes " + emergencyTimeout + "<br>" + "mainFeed " + mainFeed + " <br>" + "priceOffset " + priceOffset + " <br>" + "refreshTime " + refreshTime + " <br>" + "wallchangeTreshold " + wallchangeTreshold;
    }
}
